/**
 * Licensed to the Apache Software Foundation (ASF) under one
 * or more contributor license agreements.  See the NOTICE file
 * distributed with this work for additional information
 * regarding copyright ownership.  The ASF licenses this file
 * to you under the Apache License, Version 2.0 (the
 * "License"); you may not use this file except in compliance
 * with the License.  You may obtain a copy of the License at
 *
 *   http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an
 * "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY
 * KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations
 * under the License.
 */
package org.apache.geronimo.samples.datacdinfo.core;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;

import javax.ejb.Stateless;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.ejb.Local;
import javax.ejb.Remote;

import javax.persistence.EntityManager;
import javax.persistence.PersistenceContextType;
import javax.persistence.Query;
import javax.persistence.PersistenceContext;

import org.apache.geronimo.samples.datacdinfo.beans.DataCDBean;
import org.apache.geronimo.samples.datacdinfo.beans.OwnerBean;
import org.apache.geronimo.samples.datacdinfo.exceptions.DuplicatedDataCDException;
import org.apache.geronimo.samples.datacdinfo.exceptions.IncorrectPasswordException;
import org.apache.geronimo.samples.datacdinfo.exceptions.InvalidOwnerException;
import org.apache.geronimo.samples.datacdinfo.exceptions.InvalidPasswordException;

/**
 * 
 * Demo how to use container-managed transaction-scoped persistence context with JTA transaction.
 * 
 */
@Stateless(name="ejb/DataCDInfoJTAImpl")
@Remote(DataCDInfoRemote.class)
@Local(DataCDInfoLocal.class)
@TransactionAttribute(TransactionAttributeType.REQUIRED)
@TransactionManagement(TransactionManagementType.CONTAINER)
public class DataCDInfoJTAImpl implements DataCDInfoLocal, DataCDInfoRemote {

    @PersistenceContext(unitName = "DataCDInfoUnit", type = PersistenceContextType.TRANSACTION)
    private EntityManager em;

    public DataCDInfoJTAImpl() {
        // do nothing
    }

    public OwnerBean login(String username, String passwd)
            throws InvalidOwnerException, IncorrectPasswordException {
        // check if the owner is registered
        OwnerBean owner = em.find(OwnerBean.class, username);
        if (owner == null) {
            String error = "No such user: " + username + "!";
            throw new InvalidOwnerException(error);
        }

        if (!passwd.equals(owner.getPassword())) {
            String error = "Incorrect password!";
            throw new IncorrectPasswordException(error);
        }

        // Load CDs to cache
        populateCDs(owner);

        return owner;
    }

    public void logout(String username) {
        // do nothing

    }

    public OwnerBean register(String username, String passwd)
            throws InvalidOwnerException, InvalidPasswordException {

        OwnerBean user = em.find(OwnerBean.class, username);
        if (user != null) {
            String error = "Can not register existing user:" + username;
            throw new InvalidOwnerException(error);
        }

        if (passwd.length() < 6) {
            String error = "Invalid password:" + passwd
                    + ". Password must be at least 6 chars";
            throw new InvalidPasswordException(error);
        }

        OwnerBean newuser = new OwnerBean(username, passwd);
        try {
            em.persist(newuser);
        } catch (Exception e) {
            e.printStackTrace();
        }

        return newuser;
    }

    public boolean unregister(String username) {

        // Delete owner as well as cascadely deleting CDs owned by the owner.
        OwnerBean deluser = em.find(OwnerBean.class, username.toLowerCase());
        populateCDs(deluser);
        try {
            em.remove(deluser);
        } catch (Exception e) {
            e.printStackTrace();
        }
        return true;
    }

    public DataCDBean addCD(String ownername, String cdlabel, Integer cdsize,
            String cddesc, Date cdarchivedate)
            throws DuplicatedDataCDException, InvalidOwnerException {

        DataCDBean newcd = new DataCDBean();

        Query query = em.createNamedQuery("DataCD.findByOwnerIdandCDLabel");
        query.setParameter("ownername", ownername.toLowerCase());
        query.setParameter("cdlabel", cdlabel);
        Collection<DataCDBean> cd = query.getResultList();

        Iterator<DataCDBean> itr = cd.iterator();

        if (!itr.hasNext()) {

            newcd.setCdArchiveDate(cdarchivedate);
            newcd.setCdDescription(cddesc);
            newcd.setCdLabel(cdlabel);
            newcd.setCdSize(cdsize);

            OwnerBean theowner = em.find(OwnerBean.class, ownername
                    .toLowerCase());
            newcd.setOwner(theowner);

            try {
                em.persist(newcd);
            } catch (Exception e) {
                e.printStackTrace();
            }

        } else {
            newcd = null;
        }

        return newcd;
    }

    public Collection<DataCDBean> listCDs(String ownername) {

        OwnerBean owner = em.find(OwnerBean.class, ownername);
        Query query = em.createNamedQuery("DataCD.findByOwnerId");
        query.setParameter("ownername", owner.getUsername());
        Collection<DataCDBean> cds = query.getResultList();
        if (cds == null) {
            cds = new ArrayList<DataCDBean>();
        }
        return cds;
    }

    public boolean removeCD(Integer cdid) {
        boolean success = false;

        DataCDBean datacd = em.find(DataCDBean.class, cdid);
        if (datacd != null) {
            try {
                em.remove(datacd);
                success = true;
            } catch (Exception e) {
                e.printStackTrace();
            }
        }

        return success;
    }

    public DataCDBean updateCD(Integer cdid, Integer cdsize, String cddesc,
            Date cdarchivedate) {

        DataCDBean datacd = em.find(DataCDBean.class, cdid);
        if (datacd != null) {
            datacd.setCdArchiveDate(cdarchivedate);
            datacd.setCdDescription(cddesc);
            datacd.setCdSize(cdsize);
            try {
                em.persist(datacd);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        return datacd;
    }

    public OwnerBean refreshCDInfo(String ownername) {
        OwnerBean owner = null;
        owner = em.find(OwnerBean.class, ownername);
        if (owner != null) {
            populateCDs(owner);
        }
        return owner;
    }

    private void populateCDs(OwnerBean owner) {
        Collection<DataCDBean> ownedcds = owner.getDataCDs();
        Iterator<DataCDBean> itr = ownedcds.iterator();
        while (itr.hasNext()) {
            ((DataCDBean) itr.next()).getCdLabel();
        }
    }

}
